@@ -32,9 +32,10 @@
 #'   background process for each request.
 #'   \item Any unresolved \sQuote{mirai} will return an \sQuote{errorValue} 19
 #'   (Connection reset) after a reset.
-#'   \item Calling \code{daemons} with revised (or even the same) settings for the
-#'   same compute profile resets daemons before applying the new settings if
-#'   \code{force = TRUE}.
+#'   \item Daemons must be reset before calling \code{daemons} with revised
+#'   settings for a compute profile. Daemons may be added at any time by using
+#'   \code{\link{launch_local}} or \code{\link{launch_remote}} without needing
+#'   to revise daemons settings.
 #' }
 #'
 #' If the host session ends, all connected dispatcher and daemon processes
@@ -46,8 +47,8 @@
 #' \code{daemons(NULL)} instead, which also sends exit signals to all connected
 #' daemons prior to resetting.
 #'
-#' For historical reasons, \code{daemons()} with no arguments returns the value
-#' of \code{\link{status}}.
+#' For historical reasons, \code{daemons()} with no arguments (other than
+#' optionally \sQuote{.compute}) returns the value of \code{\link{status}}.
 #'
 #' @inheritParams mirai
 #' @inheritParams dispatcher
@@ -69,10 +70,6 @@
 #'   \code{\link{daemon}} if launching daemons. These include \sQuote{asyncdial},
 #'   \sQuote{autoexit}, \sQuote{cleanup}, \sQuote{output}, \sQuote{maxtasks},
 #'   \sQuote{idletime} and \sQuote{walltime}.
-#' @param force [default TRUE] logical value whether to always reset daemons and
-#'   apply new settings for a compute profile, even if already set. If FALSE,
-#'   applying new settings requires daemons to be explicitly reset first using
-#'   \code{daemons(0)}.
 #' @param seed [default NULL] (optional) supply a random seed (single value,
 #'   interpreted as an integer). This is used to inititalise the L'Ecuyer-CMRG
 #'   RNG streams sent to each daemon. Note that reproducible results can be
@@ -110,16 +107,15 @@
 #'
 #' By default \code{dispatcher = TRUE} launches a background process running
 #' \code{\link{dispatcher}}. Dispatcher connects to daemons on behalf of
-#' the host and ensures optimal FIFO scheduling of tasks.
+#' the host, queues tasks, and ensures optimal scheduling.
 #'
-#' Specifying \code{dispatcher = FALSE}, uses the default behaviour without
-#' additional dispatcher logic. In this case daemons connect directly to the
-#' host and tasks are distributed in a round-robin fashion. Optimal scheduling
-#' is not guaranteed as the duration of tasks cannot be known \emph{a priori},
-#' hence tasks can be queued at one daemon while other daemons remain idle.
-#' However, this provides the most resource-light approach, suited to working
-#' with similar-length tasks, or where concurrent tasks typically do not exceed
-#' available daemons.
+#' Specifying \code{dispatcher = FALSE}, daemons connect directly to the
+#' host and tasks are distributed in a round-robin fashion. As tasks are queued
+#' at each daemon, optimal scheduling is not guaranteed as the duration of each
+#' task cannot be known \emph{a priori}. Tasks can be queued at one daemon while
+#' others remain idle. However, this provides the most resource-light approach,
+#' suited to working with similar-length tasks, or where concurrent tasks
+#' typically do not exceed available daemons.
 #'
 #' @section Distributed Computing:
 #'
@@ -238,8 +234,8 @@
 #' @export
 #'
 daemons <- function(n, url = NULL, remote = NULL, dispatcher = TRUE, ...,
-                    force = TRUE, seed = NULL, serial = NULL,
-                    tls = NULL, pass = NULL, .compute = "default") {
+                    seed = NULL, serial = NULL, tls = NULL, pass = NULL,
+                    .compute = "default") {
 
   missing(n) && missing(url) && return(status(.compute))
 
@@ -264,33 +260,18 @@ daemons <- function(n, url = NULL, remote = NULL, dispatcher = TRUE, ...,
           cv <- cv()
           urld <- local_url()
           sock <- req_socket(urld)
-          res <- launch_sync_dispatcher(sock, sock, wa52(urld, dots, url), output, tls, pass, serial)
+          res <- launch_sync_dispatcher(sock, wa5(urld, dots, url), output, tls, pass, serial)
           is.object(res) && stop(._[["sync_dispatcher"]])
           store_dispatcher(sock, res, cv, envir)
           `[[<-`(envir, "msgid", 0L)
         },
-        {
-          n <- if (missing(n)) length(url) else if (is.numeric(n) && n >= 1L) as.integer(n) else stop(._[["n_one"]])
-          tls <- configure_tls(url, tls, pass, envir, returnconfig = FALSE)
-          cv <- cv()
-          urld <- local_url()
-          urlc <- sprintf("%s%s", urld, "c")
-          sock <- req_socket(urld)
-          sockc <- req_socket(urlc)
-          res <- launch_sync_dispatcher(sock, sockc, wa5(urld, dots, n, urlc, url), output, tls, pass)
-          is.object(res) && stop(._[["sync_dispatcher"]])
-          store_dispatcher(sockc, res, cv, envir)
-          launches <- n
-        },
         stop(._[["dispatcher_args"]])
       )
       `[[<-`(.., .compute, `[[<-`(`[[<-`(`[[<-`(envir, "sock", sock), "n", launches), "dots", dots))
       if (length(remote))
         launch_remote(n = n, remote = remote, tls = envir[["tls"]], ..., .compute = .compute)
-    } else if (force) {
-      daemons(0L, .compute = .compute)
-      return(daemons(n = n, url = url, remote = remote, dispatcher = dispatcher, ...,
-                     seed = seed, serial = serial, tls = tls, pass = pass, .compute = .compute))
+    } else {
+      stop(sprintf(._[["daemons_set"]], .compute))
     }
 
   } else {
@@ -305,7 +286,6 @@ daemons <- function(n, url = NULL, remote = NULL, dispatcher = TRUE, ...,
 
       if (signal) send_signal(envir)
       reap(envir[["sock"]])
-      is.null(envir[["sockc"]]) || reap(envir[["sockc"]])
       ..[[.compute]] <- NULL -> envir
 
     } else if (is.null(envir)) {
@@ -325,29 +305,17 @@ daemons <- function(n, url = NULL, remote = NULL, dispatcher = TRUE, ...,
         {
           cv <- cv()
           sock <- req_socket(urld)
-          res <- launch_sync_dispatcher(sock, sock, wa42(urld, dots, envir[["stream"]], n), output, serial = serial)
+          res <- launch_sync_dispatcher(sock, wa4(urld, dots, envir[["stream"]], n), output, serial = serial)
           is.object(res) && stop(._[["sync_dispatcher"]])
           store_dispatcher(sock, res, cv, envir)
           for (i in seq_len(n)) next_stream(envir)
           `[[<-`(envir, "msgid", 0L)
         },
-        {
-          cv <- cv()
-          sock <- req_socket(urld)
-          urlc <- sprintf("%s%s", urld, "c")
-          sockc <- req_socket(urlc)
-          res <- launch_sync_dispatcher(sock, sockc, wa4(urld, dots, envir[["stream"]], n, urlc), output)
-          is.object(res) && stop(._[["sync_dispatcher"]])
-          store_dispatcher(sockc, res, cv, envir)
-          for (i in seq_len(n)) next_stream(envir)
-        },
         stop(._[["dispatcher_args"]])
       )
       `[[<-`(.., .compute, `[[<-`(`[[<-`(`[[<-`(envir, "sock", sock), "n", n), "dots", dots))
-    } else if (force) {
-      daemons(0L, .compute = .compute)
-      return(daemons(n = n, url = url, remote = remote, dispatcher = dispatcher, ...,
-                     seed = seed, serial = serial, tls = tls, pass = pass, .compute = .compute))
+    } else {
+      stop(sprintf(._[["daemons_set"]], .compute))
     }
 
   }
@@ -451,8 +419,7 @@ status <- function(.compute = "default") {
   envir <- ..[[.compute]]
   is.null(envir) && return(list(connections = 0L, daemons = 0L))
   length(envir[["msgid"]]) && return(dispatcher_status(envir))
-  list(connections = as.integer(stat(envir[["sock"]], "pipes")),
-       daemons = if (is.null(envir[["sockc"]])) envir[["urls"]] else query_status(envir))
+  list(connections = as.integer(stat(envir[["sock"]], "pipes")), daemons = envir[["urls"]])
 
 }
 
@@ -515,13 +482,11 @@ init_envir_stream <- function(seed) {
   envir
 }
 
-tokenized_url <- function(url) sprintf("%s/%s", url, random(12L))
-
 req_socket <- function(url, tls = NULL, resend = 0L)
   `opt<-`(socket("req", listen = url, tls = tls), "req:resend-time", resend)
 
 parse_dispatcher <- function(x)
-  if (is.logical(x)) 1L + (!is.na(x) && x) else if (x == "process" || x == "thread") 3L else if (x == "none") 1L else 4L
+  if (is.logical(x)) 1L + (!is.na(x) && x) else if (x == "process" || x == "thread") 2L else if (x == "none") 1L else 3L
 
 parse_dots <- function(...) {
   ...length() || return("")
@@ -539,50 +504,47 @@ parse_tls <- function(tls)
 
 libp <- function(lp = .libPaths()) lp[file.exists(file.path(lp, "mirai"))][1L]
 
-wa31 <- function(url, dots, rs, tls = NULL)
-  shQuote(sprintf("mirai::daemon(\"%s\"%s%s,rs=c(%s))", url, dots, parse_tls(tls), paste0(rs, collapse = ",")))
+wa2 <- function(url, dots, rs, tls = NULL)
+  shQuote(sprintf("mirai::daemon(\"%s\",dispatcher=FALSE%s%s,rs=c(%s))", url, dots, parse_tls(tls), paste0(rs, collapse = ",")))
 
 wa3 <- function(url, dots, rs, tls = NULL)
-  shQuote(sprintf("mirai::daemon(\"%s\"%s%s,rs=c(%s),dispatcher=FALSE)", url, dots, parse_tls(tls), paste0(rs, collapse = ",")))
+  shQuote(sprintf("mirai::daemon(\"%s\",dispatcher=TRUE%s%s,rs=c(%s))", url, dots, parse_tls(tls), paste0(rs, collapse = ",")))
 
-wa32 <- function(url, dots, rs, tls = NULL)
-  shQuote(sprintf("mirai::daemon(\"%s\"%s%s,rs=c(%s),dispatcher=TRUE)", url, dots, parse_tls(tls), paste0(rs, collapse = ",")))
-
-wa4 <- function(urld, dots, rs, n, urlc)
-  shQuote(sprintf(".libPaths(c(\"%s\",.libPaths()));mirai::dispatcher(\"%s\",n=%d,rs=c(%s),monitor=\"%s\"%s)", libp(), urld, n, paste0(rs, collapse= ","), urlc, dots))
-
-wa42 <- function(urld, dots, rs, n)
+wa4 <- function(urld, dots, rs, n)
   shQuote(sprintf(".libPaths(c(\"%s\",.libPaths()));mirai::dispatcher(\"%s\",n=%d,rs=c(%s)%s)", libp(), urld, n, paste0(rs, collapse= ","), dots))
 
-wa5 <- function(urld, dots, n, urlc, url)
-  shQuote(sprintf(".libPaths(c(\"%s\",.libPaths()));mirai::dispatcher(\"%s\",c(\"%s\"),n=%d,monitor=\"%s\"%s)", libp(), urld, paste0(url, collapse = "\",\""), n, urlc, dots))
-
-wa52 <- function(urld, dots, url)
+wa5 <- function(urld, dots, url)
   shQuote(sprintf(".libPaths(c(\"%s\",.libPaths()));mirai::dispatcher(\"%s\",url=\"%s\"%s)", libp(), urld, url, dots))
 
 launch_daemon <- function(args, output)
   system2(.command, args = c("-e", args), stdout = output, stderr = output, wait = FALSE)
 
-launch_sync_dispatcher <- function(sock, sockc, args, output, tls = NULL, pass = NULL, serial = NULL) {
+query_dispatcher <- function(sock, command, send_mode = 2L, recv_mode = 5L, block = .limit_short)
+  if (r <- send(sock, command, mode = send_mode, block = block)) r else
+    recv(sock, mode = recv_mode, block = block)
+
+launch_sync_dispatcher <- function(sock, args, output, tls = NULL, pass = NULL, serial = NULL) {
   pkgs <- Sys.getenv("R_DEFAULT_PACKAGES")
   system2(.command, args = c("--default-packages=NULL", "--vanilla", "-e", args), stdout = output, stderr = output, wait = FALSE)
   if (is.list(serial))
     `opt<-`(sock, "serial", serial)
-  query_dispatcher(sockc, list(pkgs, tls, pass, serial), send_mode = 1L, recv_mode = 2L, block = .limit_long)
+  query_dispatcher(sock, list(pkgs, tls, pass, serial), send_mode = 1L, recv_mode = 2L, block = .limit_long)
 }
 
 launch_sync_daemons <- function(seq, sock, urld, dots, envir, output) {
   cv <- cv()
   pipe_notify(sock, cv = cv, add = TRUE)
   for (i in seq)
-    launch_daemon(wa3(urld, dots, next_stream(envir)), output)
+    launch_daemon(wa2(urld, dots, next_stream(envir)), output)
   for (i in seq)
     until(cv, .limit_long) || return(pipe_notify(sock, cv = NULL, add = TRUE))
   !pipe_notify(sock, cv = NULL, add = TRUE)
 }
 
-store_dispatcher <- function(sockc, res, cv, envir)
-  `[[<-`(`[[<-`(`[[<-`(`[[<-`(envir, "sockc", sockc), "urls", res[-1L]), "pid", as.integer(res[1L])), "cv", cv)
+store_dispatcher <- function(sock, res, cv, envir)
+  `[[<-`(`[[<-`(`[[<-`(`[[<-`(envir, "sock", sock), "urls", res[-1L]), "pid", as.integer(res[1L])), "cv", cv)
+
+sub_real_port <- function(port, url) sub("(?<=:)0(?![^/])", port, url, perl = TRUE)
 
 check_store_url <- function(sock, envir) {
   listener <- attr(sock, "listener")[[1L]]
@@ -593,26 +555,17 @@ check_store_url <- function(sock, envir) {
 }
 
 send_signal <- function(envir) {
-  signals <- max(length(envir[["urls"]]), stat(envir[["sock"]], "pipes"))
+  signals <- if (is.null(envir[["msgid"]])) stat(envir[["sock"]], "pipes") else
+    query_dispatcher(envir[["sock"]], c(0L, 0L))[1L]
   for (i in seq_len(signals)) {
     send(envir[["sock"]], ._scm_., mode = 2L)
     msleep(10L)
   }
 }
 
-query_status <- function(envir) {
-  res <- query_dispatcher(envir[["sockc"]], 0L)
-  `attributes<-`(
-    res,
-    list(
-      dim = c(envir[["n"]], 5L),
-      dimnames = list(envir[["urls"]], c("i", "online", "instance", "assigned", "complete"))
-    )
-  )
-}
-
 dispatcher_status <- function(envir) {
   status <- query_dispatcher(envir[["sock"]], c(0L, 0L))
+  is.object(status) && return(status)
   out <- list(connections = status[1L],
               daemons = envir[["urls"]],
               mirai = c(awaiting = status[2L],
